﻿#region Using directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

#endregion

namespace Loan_Assistant
{
    partial class frmLoan : Form
    {
        public frmLoan()
        {
            InitializeComponent();
        }
        bool computePayment;

        private void btnPayment_Click(object sender, EventArgs e)
        {
            // will compute payment
            computePayment = true;
            btnPayment.Visible = false;
            btnMonths.Visible = true;
            txtMonths.ReadOnly = false;
            txtMonths.TabStop = true;
            txtMonths.BackColor = Color.White;
            txtPayment.Text = "";
            txtPayment.ReadOnly = true;
            txtPayment.TabStop = false;
            txtPayment.BackColor = Color.LightYellow;
            btnCompute.Text = "Compute Monthly Payment";
            txtBalance.Focus();
        }

        private void btnMonths_Click(object sender, EventArgs e)
        {
            // will compute months
            computePayment = false;
            btnPayment.Visible = true;
            btnMonths.Visible = false;
            txtMonths.Text = "";
            txtMonths.ReadOnly = true;
            txtMonths.TabStop = false;
            txtMonths.BackColor = Color.LightYellow;
            txtPayment.ReadOnly = false;
            txtPayment.TabStop = true;
            txtPayment.BackColor = Color.White;
            btnCompute.Text = "Compute Number of Payments";
            txtBalance.Focus();
        }

        private void frmLoan_Load(object sender, EventArgs e)
        {
            btnPayment.PerformClick();
        }

        private void btnCompute_Click(object sender, EventArgs e)
        {
            double balance, interest, payment;
            int months;
            double monthlyInterest, multiplier;
            double loanBalance, finalPayment;
            if (txtBalance.Text != "")
            {
                balance = Convert.ToDouble(txtBalance.Text);
            }
            else
            {
                MessageBox.Show("Must enter a Loan Balance value.", "Input Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
                txtBalance.Focus();
                return;
            }
            if (txtInterest.Text != "")
            {
                interest = Convert.ToDouble(txtInterest.Text);
            }
            else
            {
                MessageBox.Show("Must enter an Interest Rate value.", "Input Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
                txtInterest.Focus();
                return;
            }
            monthlyInterest = interest / 1200;
            if (computePayment)
            {
                // Compute loan payment
                if (txtMonths.Text != "")
                {
                    months = Convert.ToInt32(txtMonths.Text);
                }
                else
                {
                    MessageBox.Show("Must enter a Number of Payments value.", "Input Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    txtMonths.Focus();
                    return;
                }
                if (interest == 0)
                {
                    payment = balance / months;
                }
                else
                {
                    multiplier = Math.Pow(1 + monthlyInterest, months);
                    payment = balance * monthlyInterest * multiplier / (multiplier - 1);
                }
                txtPayment.Text = String.Format("{0:f2}", payment);
            }
            else
            {
                // Compute number of payments
                if (txtPayment.Text != "")
                {
                    payment = Convert.ToDouble(txtPayment.Text);
                    if (payment <= (balance * monthlyInterest + 1.0))
                    {
                        if (MessageBox.Show("Minimum payment must be $" + String.Format("{0:f2}", (int)(balance * monthlyInterest + 1.0)) + "\r\n" + "Do you want to use the minimum payment?", "Input Error", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                        {
                            txtPayment.Text = String.Format("{0:f2}", (int)(balance * monthlyInterest + 1.0));
                            payment = Convert.ToDouble(txtPayment.Text);
                        }
                        else
                        {
                            txtPayment.Focus();
                            return;
                        }
                    }
                }
                else
                {
                    MessageBox.Show("Must enter a Monthly Payment value.", "Input Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    txtPayment.Focus();
                    return;
                }
                if (interest == 0)
                {
                    months = (int)(balance / payment);
                }
                else
                {
                    months = (int)((Math.Log(payment) - Math.Log(payment - balance * monthlyInterest)) / Math.Log(1 + monthlyInterest));
                }
                txtMonths.Text = months.ToString();
            }
            // reset payment prior to analysis to fix at two decimals
            payment = Convert.ToDouble(txtPayment.Text);
            // show analysis
            txtAnalysis.Text = "Loan Balance: $" + String.Format("{0:f2}", balance);
            txtAnalysis.Text += "\r\n" + "Interest Rate: " + String.Format("{0:f2}", interest) + "%";
            // process all but last payment
            loanBalance = balance;
            for (int paymentNumber = 1; paymentNumber <= months - 1; paymentNumber++)
            {
                loanBalance += loanBalance * monthlyInterest - payment;
            }
            // find final payment
            finalPayment = loanBalance;
            if (finalPayment > payment)
            {
                // apply one more payment
                loanBalance += loanBalance * monthlyInterest - payment;
                finalPayment = loanBalance;
                months++;
                txtMonths.Text = months.ToString();
            }
            txtAnalysis.Text += "\r\n\r\n" + Convert.ToString(months - 1) + " Payments of $" + String.Format("{0:f2}", payment);
            txtAnalysis.Text += "\r\n" + "Final Payment of: $" + String.Format("{0:f2}", finalPayment);
            txtAnalysis.Text += "\r\n" + "Total Payments: $" + String.Format("{0:f2}", (months - 1) * payment + finalPayment);
            txtAnalysis.Text += "\r\n" + "Interest Paid $" + String.Format("{0:f2}", (months - 1) * payment + finalPayment - balance);
            btnCompute.Enabled = false;
            btnNewLoan.Enabled = true;
            btnNewLoan.Focus();
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            this.Close();        
        }

        private void btnNewLoan_Click(object sender, EventArgs e)
        {
            // clear computed value and analysis
            if (computePayment)
            {
                txtPayment.Text = "";
            }
            else
            {
                txtMonths.Text = "";
            }
            txtAnalysis.Text = "";
            btnCompute.Enabled = true;
            btnNewLoan.Enabled = false;
            txtBalance.Focus();
        }

        private void txtBalance_KeyPress(object sender, KeyPressEventArgs e)
        {
            // only allow numbers, backspace, decimal point, enter
            if (e.KeyChar >= '0' && e.KeyChar <= '9')
            {
                // number values
                e.Handled = false;
            }
            else if ((int)e.KeyChar == 8)
            {
                // backspace
                e.Handled = false;
            }
            else if (e.KeyChar == '.')
            {
                // decimal point
                if (txtBalance.Text.IndexOf(".") == -1)
                {
                    // no decimal yet
                    e.Handled = false;
                }
                else
                {
                    // has decimal
                    e.Handled = true;
                }
            }
            else if ((int)e.KeyChar == 13)
            {
                // enter key – move to next box
                txtInterest.Focus();
                e.Handled = false;
            }
            else
            {
                // any other character
                e.Handled = true;
            }
        }

        private void txtInterest_KeyPress(object sender, KeyPressEventArgs e)
        {
            // only allow numbers, backspace, decimal point, enter
            if (e.KeyChar >= '0' && e.KeyChar <= '9')
            {
                // number values
                e.Handled = false;
            }
            else if ((int)e.KeyChar == 8)
            {
                // backspace
                e.Handled = false;
            }
            else if (e.KeyChar == '.')
            {
                // decimal point
                if (txtInterest.Text.IndexOf(".") == -1)
                {
                    // no decimal yet
                    e.Handled = false;
                }
                else
                {
                    // has decimal
                    e.Handled = true;
                }
            }
            else if ((int)e.KeyChar == 13)
            {
                // enter key – move to next box
                if (computePayment)
                {
                    txtMonths.Focus();
                }
                else
                {
                    txtPayment.Focus();
                }
                e.Handled = false;
            }
            else
            {
                // any other character
                e.Handled = true;
            }
        }

        private void txtMonths_KeyPress(object sender, KeyPressEventArgs e)
        {
            // only allow numbers, backspace, enter
            if (e.KeyChar >= '0' && e.KeyChar <= '9')
            {
                // number values
                e.Handled = false;
            }
            else if ((int)e.KeyChar == 8)
            {
                // backspace
                e.Handled = false;
            }
            else if ((int)e.KeyChar == 13)
            {
                // enter key – move to compute button
                btnCompute.Focus();
                e.Handled = false;
            }
            else
            {
                // any other character
                e.Handled = true;
            }
        }

        private void txtPayment_KeyPress(object sender, KeyPressEventArgs e)
        {
            // only allow numbers, backspace, decimal point, enter
            if (e.KeyChar >= '0' && e.KeyChar <= '9')
            {
                // number values
                e.Handled = false;
            }
            else if ((int)e.KeyChar == 8)
            {
                // backspace
                e.Handled = false;
            }
            else if (e.KeyChar == '.')
            {
                // decimal point
                if (txtPayment.Text.IndexOf(".") == -1)
                {
                    // no decimal yet
                    e.Handled = false;
                }
                else
                {
                    // has decimal
                    e.Handled = true;
                }
            }
            else if ((int)e.KeyChar == 13)
            {
                // enter key – move to compute button
                btnCompute.Focus();
                e.Handled = false;
            }
            else
            {
                // any other character
                e.Handled = true;
            }
        }
    }
}